MODULE_NAME='ChristieDigital_DS60_Comm' ( DEV vdvDEV, DEV dvDEV )
(*{{PS_SOURCE_INFO(PROGRAM STATS)                          *)
(***********************************************************)
(*  FILE CREATED ON: 04/15/2006 AT: 10:46:32               *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 06/23/2009  AT: 16:09:52        *)
(***********************************************************)
(*  ORPHAN_FILE_PLATFORM: 1                                *)
(***********************************************************)
(*!!FILE REVISION:                                         *)
(*  REVISION DATE: 05/19/2009                              *)
(*                                                         *)
(*  COMMENTS: Added code for use of channels 251 and 255   *)
(*            for feedback.                                *)
(*                                                         *)
(***********************************************************)
(***********************************************************)
(*}}PS_SOURCE_INFO                                         *)
(***********************************************************)

(***********************************************************)
(* System Type : Netlinx                                   *)
(***********************************************************)
(* REV HISTORY:                                            *)
(***********************************************************)

(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE
(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

VOLATILE CHAR sVERSION[] = '1.0'    // CURRENT NETLINX COMM MODULE VERSION NUMBER



(*** TIMELINE CONSTANTS ***)
INTEGER nID = 1                     // TIMELINE ID NUMBER
INTEGER nT_CNT = 1                  // TIMELINE COUNT
LONG lT_ARRAY[] = { 300000 }        // ARRAY OF TIMELINE TIMES- 5 MINUTE TIMER
(***********************************************************)
(*              DATA TYPE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE
(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

VOLATILE CHAR cBUFFER[1000]             // BUFFER FOR PHSICAL DEVICE RESPONSES
VOLATILE CHAR cCMD_QUEUE[1000]          // STORES STRINGS TO BE SENT TO THE DEVICE
VOLATILE CHAR cLAST_COMMAND[5]          // HOLDS LAST COMMAND SENT TO DEVICE
VOLATILE CHAR cDSS_MSG[20]     		// WORK AREA TO HOLD RESPONSE SENT FROM DSS
VOLATILE CHAR cTRASH[80]                // TRASH COLLECTION AREA
VOLATILE CHAR cBYTE[1]                  // ONE BYTE
VOLATILE CHAR cCMD[20]                  // COMMAND FROM UI
VOLATILE CHAR cSCMD[20]                 // COMMAND TO DS60
VOLATILE INTEGER nDEBUG = 0             // TRACKS THE ON/OFF STATE OF THE DEBUG MESSAGES GOING TO THE TELNET SESSION
VOLATILE INTEGER nFYIPOWER
VOLATILE INTEGER nPOWER = 0             // TRACKS THE CURRENT POWER STATE
VOLATILE INTEGER nINPUT = 0             // TRACKS THE CURRENT INPUT MODE
VOLATILE INTEGER nMUTE=0                // TRACKS PICTURE MUTE STATUS
VOLATILE INTEGER nCHANNEL = 0           // TRACKS THE CURRENT CHANNEL NUMBER (1-50)
VOLATILE INTEGER nWAITING = 0           // INDICATES IF WE ARE WAITING FOR A RESPONSE OR NOT
VOLATILE INTEGER nTEMP = 0              // CALCULATION AREA
VOLATILE INTEGER nBYTE	                // GET NUMBER FROM BUFFER
VOLATILE INTEGER nASPECT                // ASPECT LEVEL
VOLATILE INTEGER nBRIGHTNESS            // BRIGHTNESS LEVEL
VOLATILE INTEGER nCOLOR                 // COLOR LEVEL
VOLATILE INTEGER nCONTRAST		// CONTRAST LEVEL
VOLATILE INTEGER nTINT			// TINT LEVEL
VOLATILE SINTEGER sTEMP	                // TEMP VALUE FOR CALCULATIONS
VOLATILE CHAR DISPMSG[100]
VOLATILE INTEGER nLAMP1                 // HOURS USED LAMP 1
VOLATILE INTEGER nLAMP2                 // HOURS USED LAMP 2

(***********************************************************)
(*               LATCHING DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_LATCHING
(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE
(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)
(**************************************)
(* NAME: QUE COMMAND                  *)
(* PURPOSE: ADD A COMMAND TO THE QUE  *)
(**************************************)
DEFINE_FUNCTION QUE_CMD (CHAR cCMD[])
{
    IF ((LENGTH_STRING(cCMD_QUEUE) + LENGTH_STRING(cCMD)) <= MAX_LENGTH_ARRAY(cCMD_QUEUE))
    {
        cCMD_QUEUE = "cCMD_QUEUE,cCMD"
    }
    ELSE IF (nDEBUG) { SEND_STRING 0,"'ERRORM=Queue full - command rejected.'" }(* QUEUE IS FULL *)
    IF (!nWAITING)
    {
       SEND_CMD()
    }
}
(********************************************************)
(* SEND A COMMAND TO THE DEVICE AND WAIT FOR A RESPONSE *)
(********************************************************)
DEFINE_FUNCTION SEND_CMD()
{

    STACK_VAR INTEGER N

    IF (LENGTH_STRING(cCMD_QUEUE) >=5  AND !nWAITING)
    {
        cSCMD = REMOVE_STRING(cCMD_QUEUE,"$0D", 1)
        SEND_STRING dvDEV, "'(',cSCMD,')'"
        ON[nWAITING]
        WAIT 20 'WAIT FOR RESPONSE'
        {
            IF (nDEBUG)
	    {
		SEND_STRING 0, "'NO RESPONSE RECEIVED; SENDING NEXT CMD'"
		cSCMD = ""

	    }
            OFF[nWAITING]
//            SEND_CMD()
        }
    }
}


DEFINE_FUNCTION EVALUATE_COMMAND ()       // PROCESS ALL COMMANDS THAT HAVE VALUES 0-255
{

    SWITCH(DATA.TEXT)
    {

        CASE '+':
        {
            sTEMP=sTEMP + 1
	    IF (sTEMP > 255)
            {
		sTEMP = 255
	    }
        }

        CASE '-':
        {
            sTEMP=sTEMP - 1
	    IF (sTEMP < 0)
	    {
		sTEMP = 0
	    }
        }

        CASE '#':
        {
	    sTEMP=sTEMP + 10
	    IF (sTEMP > 255)
	    {
	        sTEMP = 255

            }
	}

        CASE '*':
        {
             sTEMP=sTEMP - 10
             IF (sTEMP < 0)
	     {
		sTEMP = 0
	     }
        }

        DEFAULT :
        {
            nTEMP=LENGTH_STRING(DATA.TEXT)
	    IF (nTEMP > 3)
	    {
	        SEND_STRING vdvDEV, "'ERRORM=Invalid ',cCMD,' value.'"
	    }
	    ELSE
	    {
		 nTEMP=ATOI(DATA.TEXT)
		 IF (nTEMP > 255)
		 {
		     SEND_STRING vdvDEV, "'ERRORM=Invalid ',cCMD,' value.'"
		 }
	    }
        }

    }

}

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

//  CREATE TIMELINE TO CAUSE A TIMELINE EVENT EVERY 10 SECONDS

TIMELINE_CREATE(nID, lT_ARRAY, nT_CNT, TIMELINE_RELATIVE, TIMELINE_REPEAT)

CREATE_BUFFER dvDEV, cBUFFER    // PHYSICAL DEVICE BUFFER
(***********************************************************)
(*                THE EVENTS GOES BELOW                    *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[dvDEV]
{
  ONLINE:
  {
        SEND_COMMAND dvDEV , 'SET BAUD 19200,N,8,1 485 DISABLED'
	ON[vdvDEV,251]
  }
  OFFLINE:
  {

	OFF[vdvDEV,251]


//       CLEAR_BUFFER cBUFFER
  }
  STRING:
  {
       IF (FIND_STRING(DATA.TEXT,'CHA!',1))
       {
	   DISPMSG=REMOVE_STRING(DATA.TEXT,'!',1)         //CHECK OF CHANNEL #
	   nCHANNEL = ATOI(DATA.TEXT)
       }

       IF (FIND_STRING(DATA.TEXT,'SIN!',1))
       {
	   DISPMSG=REMOVE_STRING(DATA.TEXT,'!',1)         //CHECK OF INPUT #
	   nINPUT = ATOI(DATA.TEXT)
       }

       IF (FIND_STRING(DATA.TEXT,'LPH!',1))
       {
	   // EXPECTING FORMAT OF LPH! "1 #### 2 ####"
	   DISPMSG=REMOVE_STRING(DATA.TEXT,'1',1)         //CHECK OF LAMP HOURS
	   nLAMP1 = ATOI(DATA.TEXT)
	   DISPMSG=REMOVE_STRING(DATA.TEXT,'2',1)         //CHECK OF LAMP HOURS
	   nLAMP2 = ATOI(DATA.TEXT)
	   SEND_LEVEL vdvDEV, 1, nLAMP1
	   SEND_LEVEL vdvDEV, 2, nLAMP2
       }

       IF (FIND_STRING(DATA.TEXT,'FYI 001',1) OR FIND_STRING(DATA.TEXT,'FYI001',1))            // IS POWER STATUS
       {
	    DISPMSG=REMOVE_STRING(DATA.TEXT,'001 ',1)
	    nFYIPOWER = ATOI(DATA.TEXT)

            SWITCH (nFYIPOWER)
            {
                CASE 0:     				  //OFF
		{
		    SEND_STRING vdvDEV, "'POWER=0'"
		    OFF[vdvDEV,255]
		}
                CASE 1:                                  //ON
		{
		    SEND_STRING vdvDEV, "'POWER=1'"
		    ON[vdvDEV,255]
		}
                CASE 10:    				 //COOLDOWN
		{
		    IF(FIND_STRING(DATA.TEXT,'Complete',1))	// HD6k PROJECTOR SENDS 2 010 COMMANDS
								// "Cooling Down" and "CoolDown Complete"
		    {
			SEND_STRING vdvDEV,"'POWER=0'"
		    }
		    ELSE
		    {

			SEND_STRING vdvDEV, "'POWER=2'"
		    }
		}
                CASE 11:                                //WARMUP
		{
		    SEND_STRING vdvDEV, "'POWER=3'"
		}
	    }
       }
       IF (FIND_STRING(DATA.TEXT,'FYI 009',1) OR FIND_STRING(DATA.TEXT,'FYI009',1))            // IS PICTURE MUTE STATUS
       {
	    DISPMSG=REMOVE_STRING(DATA.TEXT,'009 ',1)
	    nMUTE = ATOI(DATA.TEXT)

            SWITCH (nMUTE)
            {
                CASE 0:     				  //OFF
		{
		    SEND_STRING vdvDEV, "'MUTE=0'"
		}
                CASE 1:                                  //ON
		{
		    SEND_STRING vdvDEV, "'MUTE=1'"
		}  
	    }
       }	    
       DISPMSG=""
       OFF[nWAITING]
       cSCMD=''
       CANCEL_WAIT 'WAIT FOR RESPONSE'
  }

}

DATA_EVENT[vdvDEV]
{
    COMMAND:
    {
        STACK_VAR CHAR cCMD[25]
	STACK_VAR SINTEGER sTEMP

        IF (!FIND_STRING(DATA.TEXT, 'PASSTHRU=', 1))
	{
	    DATA.TEXT = UPPER_STRING(DATA.TEXT)
	}
        IF (nDEBUG)
	{
	    SEND_STRING 0, "'COMM RECEIVED FROM UI: ',DATA.TEXT"
	}
        IF(FIND_STRING(DATA.TEXT, '=', 1))
	{
	    cCMD = REMOVE_STRING(DATA.TEXT, '=', 1)
	}
        ELSE
	{
	    IF (FIND_STRING(DATA.TEXT, '?', 1))
	    {
		cCMD = REMOVE_STRING(DATA.TEXT, '?', 1)
	    }
	    ELSE
	    {
		cCMD = DATA.TEXT
	    }
	}
        SWITCH(cCMD)
        {
            // FIND MATCHING COMMAND AND PARSE REST OF MESSAGE. CONVERT MSG TO A CMD TO BE
            // SENT TO THE PHYSICAL DEVICE



            CASE 'CHANNEL=':
            {
		nTEMP = ATOI(DATA.TEXT)
		IF (nTEMP = 0 OR nTEMP > 50)
                {
                    SEND_STRING vdvDEV, "'ERRORM=Invalid CHANNEL value.'"
                }
		ELSE
		{
		    nCHANNEL = nTEMP

		    QUE_CMD("'CHA ',ITOA(nCHANNEL),13")

		}

            }


            CASE 'CHANNEL?':
            {
                QUE_CMD("'CHA?',13")
            }

            CASE 'DEBUG=':
            {
                IF (DATA.TEXT == '1' OR DATA.TEXT == '0')
                {
                    nDEBUG = ATOI(DATA.TEXT)
                    SEND_STRING vdvDEV, "'DEBUG=',ITOA(nDEBUG)"
                    SEND_STRING 0, "'DEBUG IS NOW ',ITOA(nDEBUG)"
                }
                ELSE
		{
		    SEND_STRING vdvDEV, "'ERRORM=Invalid parameter for Debug cmd.'"
		}
            }
            CASE 'DEBUG?':
	    {
		SEND_STRING vdvDEV, "'DEBUG=',ITOA(nDEBUG)"
	    }


            CASE 'INPUT=':
            {
		nTEMP = ATOI(DATA.TEXT)
		IF (nTEMP > 9)
                {
                    SEND_STRING vdvDEV, "'ERRORM=Invalid INPUT value.'"
                }
		ELSE
		{
		    nINPUT = nTEMP

                    SWITCH(nINPUT)
                    {
			CASE 1:
			{
			    QUE_CMD("'SIN 1',13")
			}
			CASE 2:
			{
			    QUE_CMD("'SIN 2',13")
			}
			CASE 3:
			{
			    QUE_CMD("'SIN 3',13")
			}
			CASE 4:
			{
			    QUE_CMD("'SIN 4',13")
			}
			CASE 5:
			{
			    QUE_CMD("'SIN 5',13")
			}
			CASE 6:
			{
			    QUE_CMD("'SIN 6',13")
			}
			CASE 7:
			{
			    QUE_CMD("'SIN 7',13")
			}
			CASE 8:
			{
			    QUE_CMD("'SIN 8',13")
			}

		    }
		}

            }

            CASE 'INPUT?':
            {

		QUE_CMD("'SIN?',13")
            }

            CASE 'MUTE=':
            {
                SWITCH(DATA.TEXT)
                {
                    CASE '0':
                    {
                        QUE_CMD("'SHU 0',13")    //PICTURE MUTE OFF
			nMUTE = 0
                    }
                    CASE '1':
                    {
                        QUE_CMD("'SHU 1',13")    // PICTURE MUTE ON
			nMUTE = 1
                    }
                    DEFAULT :
		    {
			SEND_STRING vdvDEV, "'ERRORM=Invalid value for Power cmd.'"
		    }
                }
            }
            CASE 'PASSTHRU=':
	    {
		QUE_CMD("DATA.TEXT,13")
	    }

            CASE 'POWER=':
            {
                SWITCH(DATA.TEXT)
                {
                    CASE '0':
                    {
                        QUE_CMD("'PWR 0',13")    //TURN UNIT OFF
			nPOWER = 0
                    }
                    CASE '1':
                    {
                        QUE_CMD("'PWR 1',13")    // TURN UNIT ON
			nPOWER = 1
                    }
                    DEFAULT :
		    {
			SEND_STRING vdvDEV, "'ERRORM=Invalid value for Power cmd.'"
		    }
                }
            }
            CASE 'POWER?':
            {
                 SEND_STRING vdvDEV,"'POWER=',ITOA(nPOWER)"
            }
            CASE 'VERSION?':
	    {
		SEND_STRING vdvDEV, "'VERSION=',sVERSION"
            }

            DEFAULT:
	    {
		SEND_STRING vdvDEV, "'ERRORM=Unknown Command.'"
	    }
        }
    }
}


TIMELINE_EVENT[nID]
{
    IF (nPOWER=1)
    {
	 QUE_CMD("'LPH?',13")		// REQUEST LAMP HOURS
    }

}
(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM
WAIT 5
{
    SEND_CMD ()
}
(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
